package thesis.simulation;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import thesis.gui.Galaxy;
import thesis.gui.Galaxy.Distribution;
import thesis.tree.Body;
import thesis.tree.Vector;

public final class UniverseModel {
  
  private UniverseModel() {
    
  }
  
  public static void save(List<Galaxy> galaxies, String filename)
      throws IOException {
    
    Random random = new Random();
    DataOutputStream output = new DataOutputStream(
          new BufferedOutputStream(new FileOutputStream(filename)));

    try {
      output.writeInt(galaxies.size());
      for (Galaxy galaxy : galaxies) {
        float positionX = galaxy.getPositionX().getNumber().floatValue();
        float positionY = galaxy.getPositionY().getNumber().floatValue();
        float radius = galaxy.getRadius().getNumber().floatValue();
        int numStars = galaxy.getNumberOfStars().getNumber().intValue();
        float starMass = galaxy.getMass().getNumber().floatValue() / numStars;
        Distribution distribution =
          (Distribution)galaxy.getDistibution().getSelectedItem();

        output.writeInt(numStars);
        output.writeFloat(starMass * 1000);
        output.writeFloat(galaxy.getVelocityX().getNumber().floatValue() * 30);
        output.writeFloat(galaxy.getVelocityY().getNumber().floatValue() * 30);

        for (int i = 0; i < numStars; ++i) {
          float angle = random.nextFloat() * 2 * (float)Math.PI;
          float distance = 0;
          
          if (distribution == Distribution.UNIFORM) {
            distance = random.nextFloat() * radius;
          } else if (distribution == Distribution.NORMAL) {
            do {
              distance = (float)random.nextGaussian() * radius / 3;
            } while (distance < 0 || distance >= radius);
          }
          
          output.writeFloat(positionX + distance * (float)Math.cos(angle));
          output.writeFloat(positionY + distance * (float)Math.sin(angle));
        }
      }
    } finally {
      output.close();
    }
  }
  
  public static List<Body> load(String filename) throws IOException {
    DataInputStream input = new DataInputStream(
        new BufferedInputStream(new FileInputStream(filename)));
    
    try {
      ArrayList<Body> bodies = new ArrayList<Body>();
      int galaxies = input.readInt();
      for (int i = 0; i < galaxies; ++i) {
        int numStars = input.readInt();
        float starMass = input.readFloat();
        Vector velocity = new Vector(input.readFloat(), input.readFloat());
          
        for (int j = 0; j < numStars; ++j) {
          Vector position = new Vector(input.readFloat(), input.readFloat());
          bodies.add(new Body(starMass, position, velocity));
        }
      }
      
      bodies.trimToSize();
      return bodies;
    } finally {
      input.close();
    }
  }
  
}
